window.onload = function() {
    GraphInit("poscanvasContainer", "poscanvas");
};

var debug;
////////////////////////////////////////////
// template functions of pos generation   //
////////////////////////////////////////////
function deg2rad(deg) {
    return deg * Math.PI / 180;
}

function rad2deg(rad) {
    return rad * 180 / Math.PI;
}

////////////////////////////////////////////
// AngularJS Module and Controller        //
////////////////////////////////////////////
angular.module('harktool', ['ui.bootstrap', 'ngMessages']).controller('posgenCtrl', function($scope) {

    $scope.template = "";

    console.log(" $scope=", $scope);

    $scope.isTemplateSelected = false;

    $scope.templates = {
        circle: { name: "Circle", isActive: false },
        cylinder: { name: "Cylinder", isActive: false },
        ball: { name: "Ball", isActive: false },
        grid: { name: "Grid", isActive: false }
    };

    $scope.putParams = function(template, target) {
        var postype, postypelist, outfilename;
        if (target === "mic") {
            postypelist = [
                { value: "microphone", label: "microphone" }
            ];
            outfilename = "microphones.xml";
        } else {
            postypelist = [
                { value: "tsp", label: "tsp" },
                { value: "impulse", label: "impulse" },
                { value: "soundsource", label: "soundsource" }
            ];
            outfilename = "source.xml";
        }
        postype = postypelist[0].value;

        var paramTable;
        switch (template) {
            case 'circle':
                paramTable = {
                    labelGeneral: { name: "General" },
                    ptype: { name: "Position Type", value: postype, inputtype: "list", list: postypelist },
                    path: { name: "Wav File Path", value: userhome + "filename_D{AZIMUTH:%03.1f}_E{ELEVATION:%03.1f}_R{RADIUS:%03.1f}.wav", inputtype: "text" },
                    outfile: { name: "Output File", value: outfilename, inputtype: "text", required: true },
                    channels: { name: "Use Channels", value: null, inputtype: "text", placeholder: "0,1,2,... (Empty means all channels.)", delim: ",", pattern: "^[0-9, ]*$" },

                    labelNeighbors: { name: "Neighbors" },
                    neighborsalgorithmlist: { name: "Algorithm", value: 1, inputtype: "list", list: [ { value: 1, label: "NearestNeighbor" } ] },
                    maxnumofneighbors: { name: "Maximum Number", value: 3, inputtype: "number", step: 1, min: 1 },
                    threshofneighbors: { name: "Threshold", value: 0.5, inputtype: "number", step: 0.0001, min: 0.0 },

                    labelRadius: { name: "Radius" },
                    rfrom: { name: "from", value: (target === 'mic') ? 0.0365 : 1.0, inputtype: "number", step: 0.0001, min: 0.0001 }, // TAMAGO's radius is 0.0365
                    rto: { name: "to", value: (target === 'mic') ? 0.0365 : 1.0, inputtype: "number", step: 0.0001, min: 0.0001 },
                    rinterval: { name: "interval", value: 0, inputtype: "number", step: 0.0001 },

                    labelAzimuth: { name: "Azimuth" },
                    afrom: { name: "from", value: 0, inputtype: "number" },
                    ato: { name: "to", value: 359, inputtype: "number" },
                    ainterval: { name: "interval", value: (target === 'mic') ? 45 : 5, inputtype: "number" },

                    labelElevation: { name: "Elevation" },
                    elevation: { name: "value", value: 0, inputtype: "number" },
                };
                break;
            case 'cylinder':
                paramTable = {
                    labelGeneral: { name: "General" },
                    ptype: { name: "Position Type", value: postype, inputtype: "list", list: postypelist },
                    path: { name: "Wav File Path", value: userhome + "filename_D{AZIMUTH:%03.1f}_H{HEIGHT:%03.1f}_R{RADIUS:%03.1f}.wav", inputtype: "text" },
                    outfile: { name: "Output File", value: outfilename, inputtype: "text", required: true },
                    channels: { name: "Use Channels", value: null, inputtype: "text", placeholder: "0,1,2,... (Empty means all channels.)", delim: ",", pattern: "^[0-9, ]*$" },

                    labelNeighbors: { name: "Neighbors" },
                    neighborsalgorithmlist: { name: "Algorithm", value: 1, inputtype: "list", list: [ { value: 1, label: "NearestNeighbor" } ] },
                    maxnumofneighbors: { name: "Maximum Number", value: 3, inputtype: "number", step: 1, min: 1 },
                    threshofneighbors: { name: "Threshold", value: 0.5, inputtype: "number", step: 0.0001, min: 0.0 },

                    labelRadius: { name: "Radius" },
                    rfrom: { name: "from", value: 1, inputtype: "number", step: 0.0001, min: 0.0001 },
                    rto: { name: "to", value: 1, inputtype: "number", step: 0.0001, min: 0.0001 },
                    rinterval: { name: "interval", value: 0, inputtype: "number", step: 0.0001 },

                    labelAzimuth: { name: "Azimuth" },
                    afrom: { name: "from", value: 0, inputtype: "number" },
                    ato: { name: "to", value: 359, inputtype: "number" },
                    ainterval: { name: "interval", value: (target === 'mic') ? 45 : 5, inputtype: "number" },

                    labelHeight: { name: "Height" },
                    hfrom: { name: "from", value: 0, inputtype: "number", step: 0.0001 },
                    hto: { name: "to", value: 0.1, inputtype: "number", step: 0.0001 },
                    hinterval: { name: "interval", value: 0.1, inputtype: "number", step: 0.0001 },
                };
                break;
            case 'ball':
                paramTable = {
                    labelGeneral: { name: "General" },
                    ptype: { name: "Position Type", value: postype, inputtype: "list", list: postypelist },
                    path: { name: "Wav File Path", value: userhome + "filename_D{AZIMUTH:%03.1f}_E{ELEVATION:%03.1f}_R{RADIUS:%03.1f}.wav", inputtype: "text" },
                    outfile: { name: "Output File", value: outfilename, inputtype: "text", required: true },
                    channels: { name: "Use Channels", value: null, inputtype: "text", placeholder: "0,1,2,... (Empty means all channels.)", delim: ",", pattern: "^[0-9, ]*$" },

                    labelNeighbors: { name: "Neighbors" },
                    neighborsalgorithmlist: { name: "Algorithm", value: 1, inputtype: "list", list: [ { value: 1, label: "NearestNeighbor" } ] },
                    maxnumofneighbors: { name: "Maximum Number", value: 3, inputtype: "number", step: 1, min: 1 },
                    threshofneighbors: { name: "Threshold", value: 0.5, inputtype: "number", step: 0.0001, min: 0.0 },

                    labelRadius: { name: "Radius" },
                    rfrom: { name: "from", value: 1, inputtype: "number", step: 0.0001, min: 0.0001 },
                    rto: { name: "to", value: 1, inputtype: "number", step: 0.0001, min: 0.0001 },
                    rinterval: { name: "interval", value: 0, inputtype: "number", step: 0.0001 },

                    labelAzimuth: { name: "Azimuth" },
                    afrom: { name: "from", value: 0, inputtype: "number" },
                    ato: { name: "to", value: 359, inputtype: "number" },
                    ainterval: { name: "interval", value: (target === 'mic') ? 45 : 5, inputtype: "number" },

                    labelElevation: { name: "Elevation" },
                    efrom: { name: "from", value: 0, inputtype: "number" },
                    eto: { name: "to", value: 359, inputtype: "number" },
                    einterval: { name: "interval", value: (target === 'mic') ? 45 : 10, inputtype: "number", step: 0.0001 },
                };
                break;
            case 'grid':
                paramTable = {
                    labelGeneral: { name: "General" },
                    ptype: { name: "Position Type", value: postype, inputtype: "list", list: postypelist },
                    path: { name: "Wav File Path", value: userhome + "filename_X{X_AXIS:%03.1f}_Y{Y_AXIS:%03.1f}_Z{Z_AXIS:%03.1f}.wav", inputtype: "text" },
                    outfile: { name: "Output File", value: outfilename, inputtype: "text", required: true },
                    channels: { name: "Use Channels", value: null, inputtype: "text", placeholder: "0,1,2,... (Empty means all channels.)", delim: ",", pattern: "^[0-9, ]*$" },

                    labelNeighbors: { name: "Neighbors" },
                    neighborsalgorithmlist: { name: "Algorithm", value: 1, inputtype: "list", list: [ { value: 1, label: "NearestNeighbor" } ] },
                    maxnumofneighbors: { name: "Maximum Number", value: 3, inputtype: "number", step: 1, min: 1 },
                    threshofneighbors: { name: "Threshold", value: 0.5, inputtype: "number", step: 0.0001, min: 0.0 },

                    labelGrid_X: { name: "X Axis" },
                    xfrom: { name: "from", value: (target === 'mic') ? -6 : -10, inputtype: "number", step: 0.0001 },
                    xto: { name: "to", value: (target === 'mic') ? 6 : 10, inputtype: "number", step: 0.0001 },
                    xinterval: { name: "interval", value: (target === 'mic') ? 2 : 1, inputtype: "number", step: 0.0001 },

                    labelGrid_Y: { name: "Y Axis" },
                    yfrom: { name: "from", value: (target === 'mic') ? -6 : -10, inputtype: "number", step: 0.0001 },
                    yto: { name: "to", value: (target === 'mic') ? 6 : 10, inputtype: "number", step: 0.0001 },
                    yinterval: { name: "interval", value: (target === 'mic') ? 2 : 1, inputtype: "number", step: 0.0001 },

                    labelGrid_Z: { name: "Z Axis" },
                    zfrom: { name: "from", value: 1, inputtype: "number", step: 0.0001 },
                    zto: { name: "to", value: 1, inputtype: "number", step: 0.0001 },
                    zinterval: { name: "interval", value: 0, inputtype: "number", step: 0.0001 },
                };
        }

        if (target === 'mic') {
            delete paramTable.path;
            delete paramTable.channels;
            delete paramTable.labelNeighbors;
            delete paramTable.neighborsalgorithmlist;
            delete paramTable.maxnumofneighbors;
            delete paramTable.threshofneighbors;
        }

        $scope.template = template;
        $scope.isTemplateSelected = true;

        console.log("$scope.template=", $scope.template, postype);

        $scope.posparams = paramTable;

        // set only clicked button to active
        for (var key in $scope.templates) {
            if (key === template) {
                $scope.templates[key].isActive = true;
            } else {
                $scope.templates[key].isActive = false;
            }
        }
        $scope.genposSuccess = false;
        $scope.genposError = false;
        $scope.genposErrorMsg = "";
    };

    $scope.genTemplate = function(target) {
        $scope.genposSuccess = false;
        $scope.genposError = false;
        $scope.isRunning = true;
        console.log("target=", target);
        updatePositions($scope.posparams, target);
    };

    $scope.genposSuccess = false;
    $scope.genposError = false;
    $scope.genposErrorMsg = "";
    var failed = function(msg) {
        $scope.genposSuccess = false;
        $scope.genposError = true;
        $scope.isRunning = false;
        $scope.genposErrorMsg = msg;
        $scope.$apply();
    };
    var updatePositions = function(posparams, target) {
        console.log("@@@ posparams=", posparams);

        var data = {};
        for (var key in posparams) {
            if (posparams[key].inputtype)
                data[key] = posparams[key].value;
        }
        //_.each(posparams, function(p) { data[p.id] = p.value; });
        data.template = $scope.template;

        var posxmlpath;
        var validInfo;
        ajaxGenPosXml(data)
            .then(function(res) { // genPosxml done
                posxmlpath = res.path;
                console.log("@@@@@@@@ genposxml success filename=", posxmlpath);
                return ajaxValid(target, { filename: data.outfile });
            }, function(res) { // genPosxml fail
                return Promise.reject(res);
            })
            .then(function(res) { // validxml done
                console.log("@@@@@@@@ validxml success");
                validInfo = res;
                if (target === 'mic') {
                    return Promise.resolve();
                } else {
                    return ajaxGenNeighbors({ filename: data.outfile, algorithm: data.neighborsalgorithmlist, maxnum: data.maxnumofneighbors, threshold: data.threshofneighbors });
                }
            }, function(res) { // validxml fail
                return Promise.reject(res);
            })
            .then(function(res) { // genneighbors done
                if (res)
                    console.log("@@@@@@@@ genneighbors success filename=", res.path);

                setValidSuccess(target, data.outfile, validInfo);
                GraphDraw("poscanvas", "/file/" + posxmlpath);
                $scope.genposSuccess = true;
                $scope.genposError = false;
                $scope.isRunning = false;
                $scope.$apply();
            })
            .catch(function(res) { // genneighbors fail
                var errorMsg = (res.responseText) ? res.responseText : res.statusText;
                setValidError(target, data.outfile, errorMsg);
                failed(errorMsg);
            });
    };
});
