window.onload = function() {
    GraphInit("poscanvasContainer", "poscanvas");
};

var debug;
////////////////////////////////////////////
// AngularJS Module and Controller        //
////////////////////////////////////////////
angular.module('harktool', ['ui.bootstrap', 'ngMessages', 'ui.validate']).controller('posplotCtrl', function($scope, $location) {
    console.log("PosplotMenuCtrl $scope=", $scope);
    console.log( "PosplotMenuCtrl $location=", $location);

    $scope.getFileUrl = function(filename) {
        return $location.protocol() + "://" + $location.host() + ":" + $location.port() + "/file/" + filename.trim();
    };

    $scope.putParams = function(target) {
        var postype, posChannels, postypelist;
        if (target === "mic") {
            postypelist = [
                { value: "microphone", label: "microphone" }
            ];
            $scope.filename = localStorage.getItem('micfile');
            if (localStorage.getItem('micfileinfo')) {
                postype = JSON.parse(localStorage.getItem('micfileinfo')).type;
            }
        } else {
            postypelist = [
                { value: "tsp", label: "tsp" },
                { value: "impulse", label: "impulse" },
                { value: "soundsource", label: "soundsource" }
            ];
            $scope.filename = localStorage.getItem('srcfile');
            if (localStorage.getItem('srcfileinfo')) {
                postype = JSON.parse(localStorage.getItem('srcfileinfo')).type;
                posChannels = JSON.parse(localStorage.getItem('srcfileinfo')).channels;
            }
        }

        var paramTable = {
            labelGeneral: { name: "General" },
            ptype: { name: "Position Type", value: postype, inputtype: "list", list: postypelist },
            channels: { name: "Use Channels", value: posChannels, inputtype: "text", placeholder: "0,1,2,... (Empty means all channels.)", delim: ",", pattern: "^[0-9, ]*$" },

            labelNeighbors: { name: "Neighbors" },
            neighborsalgorithmlist: { name: "Algorithm", value: 1, inputtype: "list", list: [ { value: 1, label: "NearestNeighbor" } ] },
            maxnumofneighbors: { name: "Maximum Number", value: 3, inputtype: "number", step: 1, min: 1 },
            threshofneighbors: { name: "Threshold", value: 0.5, inputtype: "number", step: 0.0001, min: 0.0 },

            labelRadius: { name: "Positions" },
        };
        if (target === 'mic') {
            delete paramTable.channels;
            delete paramTable.labelNeighbors;
            delete paramTable.neighborsalgorithmlist;
            delete paramTable.maxnumofneighbors;
            delete paramTable.threshofneighbors;
        }

        $scope.posparams = paramTable;

        $scope.positions = [];
        Plotly.d3.xml($scope.getFileUrl($scope.filename), function(error, xmlRoot) {

            var place = xmlRoot.getElementsByTagName("position");

            for(var i=0; i<place.length; i++) {
                var xmlName = place.item(i);
                var data = {};
                data.id = parseInt(xmlName.getAttribute("id"), 10); 
                data.x = xmlName.getAttribute("x");
                data.y = xmlName.getAttribute("y");
                data.z = xmlName.getAttribute("z");
                data.path = xmlName.getAttribute("path");
                $scope.positions.push(data);
            }
            $scope.$apply();
        });

        // set only clicked button to active
        $scope.genposSuccess = false;
        $scope.genposError = false;
        $scope.genposErrorMsg = "";

        GraphDraw("poscanvas", "/file/" + $scope.filename);
    };

    $scope.validatePosId = function(val,self) {
        for(var i=0; i<$scope.positions.length; i++) {
            if(i!==self && $scope.positions[i].id===val) {
                return false;
            }
        }
        return true;
    };

    ////////////////////////////////////////////////////
    ///
    /// create ボタン押下時
    ///
    $scope.updatePositions = function(target) {
        $scope.genposSuccess = false;
        $scope.genposError = false;
        $scope.isRunning = true;
        if (target === 'mic') { $scope.resetId(); }
        
        var data = {};
        for (var key in $scope.posparams) {
            if ($scope.posparams[key].inputtype)
                data[key] = $scope.posparams[key].value;
        }
        data.outfile = $scope.filename;
        _.each($scope.positions, function(row) {
            if(row.x === "-0.0000") row.x = "0.0000";
            if(row.y === "-0.0000") row.y = "0.0000";
            if(row.z === "-0.0000") row.z = "0.0000";
        });
        data.positions = $scope.positions;
        console.log(data);
        ajaxUpdatePosXml(data)
            .then(function(res) { // UpdatePosXml done
                newPosxmlpath = res.path;
                console.log("@@@@@@@@ UpdatePosXml success filename=", newPosxmlpath);
                if (target === 'mic') {
                    return Promise.resolve();
                } else {
                    return ajaxGenNeighbors({ filename: newPosxmlpath, algorithm: data.neighborsalgorithmlist, maxnum: data.maxnumofneighbors, threshold: data.threshofneighbors });
                }
            }, function(res) { // UpdatePosXml fail
                return Promise.reject(res);
            })
            .then(function(res) { // genneighbors done
                if(res)
                    console.log("@@@@@@@@ genneighbors success filename=", res.path);
                if (target === 'mic') {
                    return Promise.resolve();
                } else {
                    return ajaxMergeSrcXmls({ filename1: data.outfile, filename2: newPosxmlpath });
                }
            }, function(res) { // genneighbors fail
                return Promise.reject(res);
            })
            .then(function(res) { // mergeXmls done
                if(res)
                    console.log("@@@@@@@@ mergesrcxmls success path=", res.path);
                return ajaxValid(target, { filename: data.outfile });
            }, function(res) { // mergeXmls fail
                return Promise.reject(res);
            })
            .then(function(res) { // validxml done
                console.log("@@@@@@@@ validxml success");
                setValidSuccess(target, data.outfile, res);
                GraphDraw("poscanvas", "/file/" + data.outfile);
                $scope.genposSuccess = true;
                $scope.genposError = false;
                $scope.isRunning = false;
                $scope.$apply();
            })
            .catch(function(res) { // fail
                var errorMsg = (res.responseText) ? res.responseText : res.statusText;
                setValidError(target, data.outfile, errorMsg);
                failed(errorMsg);
            });
    };

    $scope.addRow = function() {
        var maxId = _.chain($scope.positions)
                     .map(function(pos) { return pos.id; })
                     .max() || -1;
        $scope.positions.push({id:maxId+1, x:0.0, y:0.0, z:0.0, path:""});
    };

    $scope.deleteRow = function(index) {
        $scope.positions.splice(index,1);
    };
    
    $scope.resetId = function() {
        for(var i=0; i<$scope.positions.length; i++) {
            $scope.positions[i].id = i;
        }
    };
});
